// Displayobject.h: interface for the CBaseObject class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_DISPLAYOBJECT_H__D660E270_0FB3_431B_A4A7_B611CD552C17__INCLUDED_)
#define AFX_DISPLAYOBJECT_H__D660E270_0FB3_431B_A4A7_B611CD552C17__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

//directx
#include <d3d8.h> // LINK d3d8.lib
#include <d3dx8.h> // LINK d3dx8.lib

// the base class for all display objects
class CBaseObject  
{
public:
	CBaseObject();

	CBaseObject* m_pNext;          // Link to next object
    CBaseObject* m_pPrev;          // Link to previous object
    
    DWORD          m_dwType;            // Type of object
    BOOL           m_bVisible;          // Whether the object is visible
    D3DXVECTOR3    m_vPos;              // Position
	// these two vector denotes the orientation of the object above the terrain
	// actually this should be a normal vector, but two scale number is suffient for this game
    float m_fAngleX, m_fAngleY;
	
	float   m_fVel;              // Velocity
   	float	m_fHeight; // the flying height of the object
	DWORD	m_dwMark; // the mark generated by the controller

	float	m_fSize;				// radius of the object

	virtual ~CBaseObject();
};

// classes that derived from CBaseObject

enum {MSIL_NORMAL, MSIL_CANNON, MSIL_TRACK, EXPLOSION_LARGE, PLANE_USER, PLANE_ENEMY1, PLANE_ENEMY2};

/////////////////// mark constants///////////////
// PLANE MARKS
#define	MARK_LEFT		1
#define	MARK_RIGHT		2
#define	MARK_FORWARD	4
#define	MARK_BACKWARD	8
#define	MARK_GLIDING	16
#define	MARK_FIRE		32

// GENERAL MARKS
#define	MARK_DEAD		64

#define GET_MARK(mark, item)	((mark) & (item))
#define CLEAR_PLANEMARK(mark)	((mark) & 0xffffffc0)

////////////////////////////////////////////////////
// one kind of the user planes 
class CPlane : public CBaseObject  
{
public:
	int m_nHealth;
	// Bullet mode /////////////////////////////////////////////////////////
	DWORD m_fRechargeTime;
	DWORD m_fLastFireTime;//  = 0.0f;  
	DWORD m_dwBulletType;// = 0L;    // Current bullet type
	float m_fAngle;	// when the plane is turning, this value will increase/decrease from 0.0f
	DWORD                m_dwCurrentShipType;// = 0L; ship model being used


	CPlane();
	virtual ~CPlane();

};

// one of the bullets
enum { USER, ENEMY};
class CMissile : public CBaseObject  
{
public:
	int m_nAttackPoint;
	BYTE m_nManufactory; // who manufactured the missile the enemy planes or the user
	D3DXVECTOR3 m_vTarget;
		
	D3DXVECTOR3 m_fTarget;
	float m_fRange; // how far a bullet can travel
	float m_fTransition;// = 0.0f;  // Amount used to transition tracking(in percentage)
	CMissile();
	virtual ~CMissile();

};

// one kind of the enemies
class CEnemy : public CPlane
{
public:
	CEnemy();
	virtual ~CEnemy();

};


class CTrackMissile : public CMissile  
{
public:
	CTrackMissile();
	virtual ~CTrackMissile();

};

#endif // !defined(AFX_DISPLAYOBJECT_H__D660E270_0FB3_431B_A4A7_B611CD552C17__INCLUDED_)
